"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const uuid = require("uuid");
const replyQueue_1 = require("./replyQueue");
const race_until_1 = require("race-until");
const log4js = require("@log4js-node/log4js-api");
const logger = log4js.getLogger('rabbit-queue');
class Queue {
    constructor(channel, name, options) {
        this.channel = channel;
        this.name = name;
        this.defaultOptions = {
            durable: true,
            noAck: false
        };
        this.options = Object.assign({}, this.defaultOptions, options);
        this.created = this.create();
    }
    create() {
        return __awaiter(this, void 0, void 0, function* () {
            const { exclusive, priority, durable, autoDelete, messageTtl, expires, deadLetterExchange, deadLetterRoutingKey, maxLength } = this.options;
            let queueOptions = {
                exclusive,
                durable,
                autoDelete,
                messageTtl,
                expires,
                deadLetterExchange,
                deadLetterRoutingKey,
                maxLength
            };
            if (priority !== undefined) {
                queueOptions.arguments = { 'x-max-priority': priority };
            }
            yield this.channel.assertQueue(this.name, queueOptions);
        });
    }
    subscribe(handler) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.created;
            this.handler = handler;
            let tag = yield this.channel.consume(this.name, this.onMessage.bind(this), { noAck: this.options.noAck });
            this.tag = tag;
        });
    }
    unsubscribe() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.channel.cancel(this.tag.consumerTag);
            this.handler = null;
            this.tag = null;
        });
    }
    static destroy(channel, name) {
        return __awaiter(this, void 0, void 0, function* () {
            yield channel.deleteQueue(name);
        });
    }
    purge() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.channel.purgeQueue(this.name);
        });
    }
    onMessage(msg) {
        const ack = () => {
            if (!this.options.noAck) {
                this.channel.ack(msg);
            }
        };
        if (!msg) {
            return;
        }
        const hasReply = !!msg.properties.replyTo;
        this.handler(msg, (error, reply) => {
            if (hasReply && reply !== Queue.STOP_PROPAGATION) {
                if (error) {
                    reply = Object.assign({}, Queue.ERROR_DURING_REPLY, { error_message: error });
                }
                var replyBuffer = Buffer.from(JSON.stringify(reply || ''));
                this.channel.sendToQueue(msg.properties.replyTo, replyBuffer, {
                    correlationId: msg.properties.correlationId
                }, ack);
            }
            else {
                ack();
            }
        });
    }
    static publish(obj, headers = {}, channel, name, queue) {
        return __awaiter(this, void 0, void 0, function* () {
            if (queue) {
                yield queue.created;
            }
            return new Promise((resolve, reject) => {
                var msg = JSON.stringify(obj);
                var correlationId = headers.correlationId || uuid.v4();
                headers = Object.assign({
                    persistent: true,
                    correlationId
                }, headers);
                const bufferContent = Buffer.from(msg);
                logger.info(`[${correlationId}] -> Publishing to queue ${name} ${bufferContent.byteLength} bytes`);
                channel.sendToQueue(name, bufferContent, headers, (err, ok) => (err ? reject(err) : resolve(ok)));
            });
        });
    }
    static getReply(obj, headers, channel, name, queue, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            if (queue) {
                yield queue.created;
            }
            const reply = replyQueue_1.getReply(obj, headers, channel, (bufferContent, headers, correlationId, cb) => {
                logger.info(`[${correlationId}] -> Publishing to reply queue ${name} ${bufferContent.byteLength} bytes`);
                channel.sendToQueue(name, bufferContent, headers, cb);
            });
            if (timeout) {
                return race_until_1.default(reply, timeout, false);
            }
            else {
                return reply;
            }
        });
    }
    static bindToExchange(exchange, routingKey, channel, name, queue) {
        return __awaiter(this, void 0, void 0, function* () {
            if (queue) {
                yield queue.created;
            }
            yield channel.bindQueue(name, exchange, routingKey);
            logger.debug(`created binding ${exchange} routingkey:${routingKey} --> queue:${name}`);
        });
    }
    static unbindFromExchange(exchange, routingKey, channel, name, queue) {
        return __awaiter(this, void 0, void 0, function* () {
            if (queue) {
                yield queue.created;
            }
            yield channel.unbindQueue(name, exchange, routingKey);
            logger.debug(`deleted binding${exchange} routingkey:${routingKey} -X-> queue:${name}}`);
        });
    }
}
Queue.STOP_PROPAGATION = { stopPropagation: true };
Queue.ERROR_DURING_REPLY = { error: true, error_code: 999 };
exports.default = Queue;
//# sourceMappingURL=queue.js.map